from flask import Flask, request, jsonify, render_template, redirect, url_for
from flask_sqlalchemy import SQLAlchemy
import requests
import os
 
app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///gitlab_status_checks.db'
db = SQLAlchemy(app)
print("new version wit merge request id ")
# Define the StatusCheck model
class StatusCheck(db.Model):
    __tablename__ = 'status_check'
    id = db.Column(db.Integer, primary_key=True)
    merge_request_id = db.Column(db.Integer, nullable=True)
    sha = db.Column(db.String(40), nullable=True)
    status = db.Column(db.String(10), nullable=True)
    description = db.Column(db.String(255), nullable=True)
    target_url = db.Column(db.String(255), nullable=True)
    project_id = db.Column(db.Integer, nullable=False)
    external_approval_rule = db.Column(db.Integer, nullable=False)
 
  
# Create the database
with app.app_context():
    try:
        db.drop_all()
        db.create_all()
        print("Database initialized.123")
    except Exception as e:
        print(f"Error creating tables: {e}")
    print("Database initialized.")

# Route to receive status check requests from GitLab
@app.route('/approval-apps/status_check', methods=['GET', 'POST'])
def status_check():
    data = request.json
    print(data)
    #merge_request_id = data['merge_request_id']
    merge_request_id = data.get('object_attributes', {}).get('iid')
    print("New Hello")
    print(f"merge_request_id issss: {merge_request_id}")
    #sha = data['merge_commit_sha']
    #sha = data.get('object_attributes', {}).get('merge_commit_sha')
    sha = data.get('object_attributes', {}).get('last_commit', {}).get('id')
    print(f"sha: {sha}")
    #project_id = data['project_id']
    project_id = data.get('project', {}).get('id')
    #description = data['description']
    #target_url = data['target_url']
    external_status_check_id = data.get('external_approval_rule', {}).get('id')
    print(f"external_status_check_id: {external_status_check_id}")
    
    # Store the received data in the database
    new_check = StatusCheck(sha=sha, status='pending', project_id=project_id, merge_request_id=merge_request_id,external_approval_rule=external_status_check_id)
    db.session.add(new_check)
    db.session.commit()
    
    return jsonify({'message': 'Status check received'}), 200

# Route to display the status checks and approve/reject buttons
# @app.route('/')
@app.route("/approval-apps/", methods=['GET', 'POST'] ) 
def index():
    # Print the full URL of the incoming request
    print(f"Request URL: {request.url}")

    # Print the path of the incoming request
    print(f"Request Path: {request.path}")
    checks = StatusCheck.query.all()
    return render_template('index.html', checks=checks)

# Route to handle approval or rejection of a status check
@app.route('/update_status/<int:check_id>/<string:action>', methods=['POST'])
def update_status(check_id, action):
    check = StatusCheck.query.get_or_404(check_id)
    
    # Update the status based on the action
    if action == 'approve':
        check.status = 'passed'
        check.description = 'Approved by user'
    elif action == 'reject':
        check.status = 'failed'
        check.description = 'Rejected by user'
    
    # Send the updated status back to GitLab
    gitlab_token = os.getenv('GITLAB_TOKEN')  # Set GitLab token as an environment variable
    #headers = {'PRIVATE-TOKEN': gitlab_token}
    headers = {
        "PRIVATE-TOKEN": gitlab_token,
        "Content-Type": "application/json"
    }
    # print gitlab_token 



    status_update = {
        'sha': check.sha,
        'status': check.status,
        'external_status_check_id': check.external_approval_rule
    }
    print(f"GitLab Token: {gitlab_token}")
    print(f"check.project_id: {check.project_id}")
    print(f"check.merge_request_id: {check.merge_request_id}")
    print(f"check.status: {check.status}")
    print(f"check.description: {check.description}")
    print(f"check.target_url: {check.target_url}")
    print(f"check.external_approval_rule:------ {check.external_approval_rule}")
    print(f"sha: {check.sha}")
    print(f"status_update: {check.status}")
    print(f"https://gitlab.com/api/v4/projects/{check.project_id}/merge_requests/{check.merge_request_id}/status_check_responses")

     #'https://gitlab.com/api/v4/projects/{check.project_id}/statuses/{check.sha}',
    response = requests.post(
        f'https://gitlab.com/api/v4/projects/{check.project_id}/merge_requests/{check.merge_request_id}/status_check_responses',
        headers=headers,
        json=status_update
    )
    
    if response.status_code == 201:
        db.session.commit()
        return redirect(url_for('index'))
    else:
        return jsonify({'error': 'Failed to update status with GitLab'}), 400


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000,debug=True)